@@ -555,7 +555,7 @@ tga_palette_read(io_glue *ig, i_img *img, int bytepp, int colourmaplength) {
   unsigned char *palbuf;
   i_color val;
 
-  palbsize = colourmaplength*bytepp;
+  palbsize = (size_t)colourmaplength * bytepp;
   palbuf   = mymalloc(palbsize);
   
   if (i_io_read(ig, palbuf, palbsize) != palbsize) {
@@ -723,8 +723,8 @@ i_readtga_wiol(io_glue *ig, int length) {
   
   mapped = 1;
   switch (header.datatypecode) {
-  case 2:  /* Uncompressed, rgb images          */
-  case 10: /* Compressed,   rgb images          */
+  case 2:  /* Uncompressed, rgb images          */ /* FALLTHROUGH */
+  case 10: /* Compressed,   rgb images          */ /* FALLTHROUGH */
     mapped = 0;
   case 1:  /* Uncompressed, color-mapped images */
   case 9:  /* Compressed,   color-mapped images */
@@ -736,7 +736,7 @@ i_readtga_wiol(io_glue *ig, int length) {
     if (idstring) myfree(idstring);
     return NULL;
     break;
-  case 3:  /* Uncompressed, grayscale images    */
+  case 3:  /* Uncompressed, grayscale images    */ /* FALLTHROUGH */
   case 11: /* Compressed,   grayscale images    */
     mapped = 0;
     channels = 1;
@@ -844,7 +844,7 @@ i_writetga_wiol(i_img *img, io_glue *ig, int wierdpack, int compress, char *idst
 
   mm_log((1,"i_writetga_wiol(img %p, ig %p, idstring %p, idlen %ld, wierdpack %d, compress %d)\n",
 	  img, ig, idstring, (long)idlen, wierdpack, compress));
-  mm_log((1, "virtual %d, paletted %d\n", img->virtual, mapped));
+  mm_log((1, "virtual %d, paletted %d\n", i_img_virtual(img), mapped));
   mm_log((1, "channels %d\n", img->channels));
   
   i_clear_error();
@@ -918,7 +918,7 @@ i_writetga_wiol(i_img *img, io_glue *ig, int wierdpack, int compress, char *idst
   if (img->type == i_palette_type) {
     if (!tga_palette_write(ig, img, bitspp, i_colorcount(img))) return 0;
     
-    if (!img->virtual && !dest.compressed) {
+    if (!i_img_virtual(img) && !dest.compressed) {
       if (i_io_write(ig, img->idata, img->bytes) != img->bytes) {
 	i_push_error(errno, "could not write targa image data");
 	return 0;
